import requests
import time

class APIResponseError(Exception):
    """ Exception raise if the API replies with an HTTP code
    not in the 2xx range. """
    pass

class Session:
    def __init__(self):
        self.__requests_session = requests.Session()
        # self.__requests_session.headers.update({"User-Agent": "medex"})

    def api_request(self, url, params=None, auth=None, http_call="get"):
        if http_call == "get":
            response = self.__requests_session.get(url)
        elif http_call == "post":
            response = self.__requests_session.post(url, data=params, auth=auth)
        
        if not response.ok:
            if response.status_code == 429:
                time.sleep(15)
            msg = "Status Code: {}".format(response.status_code)
            raise APIResponseError(msg)
        else:
            response = response.json()
        
        if "error" in response:
            msg = "Error: {}".format(response["error"])
            if "error_description" in response:
                msg = "{}; Description: {}".format(msg, response["error_description"])
            if response["error"] == "too_many_requests":
                time.sleep(15)
            raise APIResponseError(msg)
        
        return response



from requests.auth import AuthBase
from urllib.parse import urlencode
import hashlib
import hmac
import time

from . import common

class IndodaxAuth(AuthBase):
    def __init__(self, key, sign):
        # setup any auth-related data here
        self.key = key
        self.sign = sign
    def __call__(self, r):
        # modify and return the request
        r.headers["Key"] = self.key
        r.headers["Sign"] = self.sign
        return r

def nonce():
    time.sleep(1/1000)
    return str(int(time.time()*1000))

def signature(secret, params):
    sig = hmac.new(secret.encode(), params.encode(), hashlib.sha512)
    return sig.hexdigest()


class TradeAPI:
    def __init__(self, key, secret, requests_session=None):
        self.__key = key
        self.__secret = secret

        if requests_session is not None:
            self.__requests_session = requests_session
        else:
            self.__requests_session = common.Session()

    def __post(self, method, params):
        url = "https://indodax.com/tapi"
        params["method"] = method
        params["nonce"] = nonce()
        auth = IndodaxAuth(self.__key, signature(self.__secret, urlencode(params)))
        response = self.__requests_session.api_request(url, params, auth, "post")

        return response["return"]

    def getInfo(self):
        return self.__post("getInfo", {})

    def transHistory(self):
        return self.__post("transHistory", {})

    def trade(self, pair, ttype, amount, price):
        params = {
            "pair" : pair,
            "type" : ttype,
            "price" : price}
        if ttype == "buy":
            params[pair[-3:]] = amount
        elif ttype == "sell":
            params[pair[:3]] = amount
        return self.__post("trade", params)

    def tradeHistory(self, pair, **kwargs):
        """Keyword arguments : count, from_id, end_id, order, since, end"""
        params = {
            "pair" : pair
        }
        if kwargs:
            for key, value in kwargs.items():
                params[key] = value
        return self.__post("tradeHistory", params)

    def openOrders(self, pair):
        params = { "pair" : pair }
        return self.__post("openOrders", params)

    def orderHistory(self, pair, **kwargs):
        """Keyword arguments : count, from"""
        params = {
            "pair" : pair
        }
        if kwargs:
            for key, value in kwargs.items():
                params[key] = value
        return self.__post("orderHistory", params)

    def getOrder(self, pair, order_id):
        params = {
            "pair" : pair,
            "order_id" : order_id
        }
        return self.__post("getOrder", params)

    def cancelOrder(self, pair, ttype, order_id):
        params = {
            "pair" : pair,
            "order_id" : order_id,
            "type" : ttype}
        return self.__post("cancelOrder", params)



import pandas as pd

from . import common

def get_data(pair, param, requests_session):
    if requests_session is None:
        requests_session = common.Session()

    url = "https://indodax.com/api/"+pair+"/"+param

    response = requests_session.api_request(url)
    
    return response


def getTicker(pair="btc_idr", session=None):
    """
    Retrieve the ticker for the given pair.  Returns a Ticker instance.

    Arguments:
    pair : trading pair
    session : indodax.Session object
    """

    response = get_data(pair, "ticker", requests_session=session)

    ticker = {}
    for s in ("high", "low", "vol_idr", "last", "buy", "sell", "server_time"):
        ticker[s] = int(response["ticker"].get(s))
    vol_base = "vol_" + pair[:3]
    vol_counter = "vol_" + pair[-3:]
    for s in (vol_base, vol_counter):
        ticker[s] = float(response["ticker"].get(s))

    return ticker


def getDepth(pair="btc_idr", session=None):
    """
    Retrieve the depth for the given pair.  Returns a dictionary of asks and bids dataframe.
    
    Arguments:
    pair : trading pair
    session : indodax.Session object
    """

    depth = get_data(pair, "depth", requests_session=session)

    asks = pd.DataFrame(depth["sell"])
    asks.rename(columns={0:"price",1:"volume"},inplace=True)
    asks[["price", "volume"]] = asks[["price", "volume"]].apply(pd.to_numeric)

    bids = pd.DataFrame(depth["buy"])
    bids.rename(columns={0:"price",1:"volume"},inplace=True)
    bids[["price", "volume"]] = bids[["price", "volume"]].apply(pd.to_numeric)

    return {"Asks": asks, "Bids": bids}


def getTradeHistory(pair="btc_idr", session=None):
    """
    Retrieve the trade history for the given pair.  Returns a pandas dataframe.
    
    Arguments:
    pair : trading pair
    session : indodax.Session object
    """

    history = get_data(pair, "trades", requests_session=session)

    df = pd.DataFrame(history)
    df[["date" , "price", "amount", "tid"]] = df[["date" , "price", "amount", "tid"]].apply(pd.to_numeric)
    df.set_index(df.tid.values , drop=False, inplace=True)
    df = df.reindex(columns=["tid", "date", "price", "amount", "type"])

    return df



from setuptools import setup
setup(
    name = "python-indodax", 
    packages = ["indodax"], 
    version = "0.0.1", 
    license="MIT", 
    description = "Python wrapper for indodax.com API", 
    author = "Faisal Malik Widya Prasetya", 
    author_email = "faisalmalikwidyaprasetya@gmail.com", 
    url = "https://github.com/MasiCal354/python-indodax", 
    download_url = "https://github.com/MasiCal354/python-indodax/archive/v_01.tar.gz", 
    keywords = ["indodax", "vipbtc", "api"], 
    install_requires=["requests","pandas"],
    classifiers=[
        "Development Status :: 3 - Alpha", 
        "Intended Audience :: Developers", 
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: MIT License", 
        "Programming Language :: Python :: 3", 
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9"
    ]
)